# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('djnotty', '0008_auto_20160118_1730'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messagetouser',
            name='message',
            field=models.ForeignKey(related_name='message_to_user', to='djnotty.Message'),
        ),
        migrations.AlterField(
            model_name='messagetouser',
            name='user',
            field=models.ForeignKey(related_name='messages', to=settings.AUTH_USER_MODEL),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.db import models, migrations


def clear_messages(apps, schema_editor):
    m = apps.get_model('djnotty', 'Message')
    print m.objects.count()
    m.objects.all().delete()
    print m.objects.count()


class Migration(migrations.Migration):
    dependencies = [
        ('djnotty', '0007_message_builders'),
    ]

    operations = [
        migrations.RunPython(clear_messages)
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djnotty', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='message',
            name='globally',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='message',
            name='mark_as_viewed_at',
            field=models.DateTimeField(null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djnotty', '0004_auto_20160113_1614'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='message',
            name='noty_options',
        ),
        migrations.RemoveField(
            model_name='messagetogroup',
            name='viewed',
        ),
        migrations.AddField(
            model_name='message',
            name='script',
            field=models.TextField(null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djnotty', '0003_auto_20160113_1601'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='message',
            name='driver',
        ),
        migrations.RemoveField(
            model_name='message',
            name='html',
        ),
        migrations.AddField(
            model_name='message',
            name='noty_options',
            field=jsonfield.fields.JSONField(null=True, blank=True),
        ),
    ]



# coding: utf-8
from __future__ import unicode_literals
from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.conf import settings
from jsonfield import JSONField


class MessageObjects(models.Manager):
    def make_message(self, msg, builders, globally=False, users=None, groups=None):
        if users or groups:
            globally = False
        noty_opts = {}
        scripts = []
        for b in builders:
            noty_opts = b.make(noty_opts)
            if hasattr(b, 'script_name'):
                scripts.append(b.script_name)

        msg.noty_options = noty_opts
        msg.globally = globally
        msg.builders = ','.join(scripts)
        msg.save()
        if users:
            for u in users:
                MessageToUser.objects.create(user=u, message=msg)

        if groups:
            for g in groups:
                MessageToGroup.objects.create(group=g, message=msg)
        return msg

    def create_for_object(self, model, **kwargs):
        cnt = ContentType.objects.get_for_model(model)
        msg = Message()

        msg.content_type = cnt
        msg.object_id = model.pk
        print kwargs
        self.make_message(msg, **kwargs)
        msg.save()

    def create_globally(self, builders):
        return self.make_message(Message(), builders, globally=True)

    def mark_as_viewed_for_object(self, model, user=None):
        cnt = ContentType.objects.get_for_model(model)
        if user:
            MessageToUser.objects.filter(message__content_type=cnt, message__object_id=model.pk, user=user).update(viewed=True)
        else:
            Message.objects.filter(content_type=cnt, object_id=model.pk).update(viewed=True)


class Message(models.Model):
    viewed = models.BooleanField(default=False)
    groups = models.ManyToManyField(Group, through='MessageToGroup')
    users = models.ManyToManyField(settings.AUTH_USER_MODEL, through='MessageToUser')
    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    mark_as_viewed_at = models.DateTimeField(blank=True, null=True, editable=True)
    globally = models.BooleanField(default=False)

    content_type = models.ForeignKey(ContentType, null=True, blank=True)
    object_id = models.PositiveIntegerField(null=True, blank=True)
    builders = models.CharField(max_length=500, blank=True)
    noty_options = JSONField(blank=True, null=True)
    objects = MessageObjects()

    class Meta:
        index_together = (('content_type', 'object_id'))


class MessageToUser(models.Model):
    message = models.ForeignKey(Message, related_name='message_to_user')
    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='messages')
    viewed = models.BooleanField(default=False)


class MessageToGroup(models.Model):
    message = models.ForeignKey(Message)
    group = models.ForeignKey(Group)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djnotty', '0002_auto_20160113_1412'),
    ]

    operations = [
        migrations.AddField(
            model_name='message',
            name='content_type',
            field=models.ForeignKey(blank=True, to='contenttypes.ContentType', null=True),
        ),
        migrations.AddField(
            model_name='message',
            name='object_id',
            field=models.PositiveIntegerField(null=True, blank=True),
        ),
        migrations.AlterIndexTogether(
            name='message',
            index_together=set([('content_type', 'object_id')]),
        ),
        migrations.RemoveField(
            model_name='message',
            name='title',
        ),
    ]



# coding: utf-8
from __future__ import unicode_literals
from django.core.urlresolvers import reverse


class Builder(object):
    def make(self):
        raise NotImplementedError()


class Linked(Builder):
    script_name = 'djnotty_linked'

    def __init__(self, url):
        self.url = url

    def make(self, opts):
        opts.update({'linked': self.url})
        return opts


class Close(Builder):
    script_name = 'djnotty_close'

    def __init__(self, url=None):
        self.url = url

    def make(self, opts):
        opts.update({'url': reverse('djnotty:close')})
        if self.url:
            opts['linked'] = self.url

        return opts


class Text(Builder):
    def __init__(self, text):
        self.text = text

    def make(self, opts):
        opts.update({'text': self.text})

        return opts



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        (b'auth', b'__first__'),
        (b'contenttypes', b'__first__'),
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=200)),
                ('html', models.TextField()),
                ('viewed', models.BooleanField(default=False)),
                ('driver', models.CharField(max_length=200, blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='MessageToGroup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('viewed', models.BooleanField(default=False)),
                ('group', models.ForeignKey(to='auth.Group')),
                ('message', models.ForeignKey(to='djnotty.Message')),
            ],
        ),
        migrations.CreateModel(
            name='MessageToUser',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('viewed', models.BooleanField(default=False)),
                ('message', models.ForeignKey(to='djnotty.Message')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='message',
            name='groups',
            field=models.ManyToManyField(to='auth.Group', through='djnotty.MessageToGroup'),
        ),
        migrations.AddField(
            model_name='message',
            name='users',
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL, through='djnotty.MessageToUser'),
        ),
    ]



from django.contrib import admin
from djnotty import models


class MessageToUserInline(admin.TabularInline):
    model = models.MessageToUser


class MessageToGroupInline(admin.TabularInline):
    model = models.MessageToGroup


@admin.register(models.Message)
class MessageAdmin(admin.ModelAdmin):
    inlines = [MessageToUserInline,MessageToGroupInline]
